@@ -26,6 +26,7 @@
 #include <qvbox.h>
 #include <qlabel.h>
 #include <qtooltip.h>
+#include <qtoolbutton.h>
 
 #include <kglobal.h>
 #include <klocale.h>
@@ -59,6 +60,7 @@
 #include <torrent/uploadcap.h>
 #include <torrent/downloadcap.h>
 #include <util/error.h>
+#include <torrent/serverauthenticate.h>
 #include <torrent/globals.h>
 #include <torrent/tracker.h>
 #include <torrent/downloader.h>
@@ -86,11 +88,14 @@
 #include <interfaces/functions.h>
 #include <interfaces/plugin.h>
 #include <interfaces/prefpageinterface.h>
-#include <expandablewidget.h>
+#include <newui/dtabwidget.h>
 #include <pluginmanager.h>
 #include <groups/group.h>
 #include <groups/groupview.h>
+#include <groups/groupmanager.h>
 #include <mse/streamsocket.h>
+#include "viewmanager.h"
+#include "ktorrentviewitem.h"
 
 
 
@@ -107,82 +112,68 @@ using namespace bt;
 using namespace kt;
 
 KTorrent::KTorrent()
-	: KMdiMainFrm( 0, "KTorrent",KMdi::IDEAlMode ),m_group_view(0),
-		m_view(0), m_systray_icon(0)
+	: DMainWindow(0,"KTorrent"),m_group_view(0),
+		m_view_man(0), m_systray_icon(0),m_status_prog(0)
 {
 	setHidden(true);
-	setToolviewStyle(KMdi::TextAndIcon);
+	//setToolviewStyle(KMdi::TextAndIcon);
+	connect(this,SIGNAL(widgetChanged(QWidget*)),this,SLOT(currentTabChanged(QWidget*)));
 	
-	KIconLoader* iload = KGlobal::iconLoader();
+	m_view_man = new ViewManager(this);
+	m_group_view = new kt::GroupView(m_view_man,actionCollection());
+	connect(m_group_view,SIGNAL(openNewTab(kt::Group*)),this,SLOT(openView(kt::Group*)));
 	
-	m_view = new KTorrentView(0);
-	m_view_exp = new ExpandableWidget(m_view,0);
-	addTabPage(m_view_exp,iload->loadIconSet("folder", KIcon::Small),i18n("Torrents"));
-	
-	m_group_view = new kt::GroupView(m_view,actionCollection());
-	m_group_view->loadGroups();
 
 	m_pref = new KTorrentPreferences(*this);
+	
 	m_core = new KTorrentCore(this);
+	m_core->setGroupManager(m_group_view->groupManager());
+	
 	m_systray_icon = new TrayIcon(m_core, this);
 	
+	m_group_view->loadGroups();
+	m_view_man->restoreViewState(KGlobal::config(),this);
+	
+	QToolButton* tb = new QToolButton(m_activeTabWidget);
+	tb->setIconSet(SmallIcon("tab_new"));
+	tb->adjustSize();
+	connect(tb,SIGNAL(clicked()),this,SLOT(openDefaultView()));
+	m_activeTabWidget->setCornerWidget(tb, TopLeft);
+	
 	connect(m_group_view,SIGNAL(currentGroupChanged( kt::Group* )),
 			this,SLOT(groupChanged(kt::Group*)));
 	
 	connect(m_group_view,SIGNAL(groupRenamed(kt::Group*)),
-			this,SLOT(groupChanged(kt::Group*)));
+			this,SLOT(groupRenamed(kt::Group*)));
+	
+	connect(m_group_view,SIGNAL(groupRemoved(kt::Group*)),
+			this,SLOT(groupRemoved(kt::Group*)));
 	
-	connect(m_core, SIGNAL(finished( kt::TorrentInterface* )), 
-			m_view, SLOT(torrentFinished( kt::TorrentInterface* )));
-
 	connect(m_core,SIGNAL(torrentAdded(kt::TorrentInterface* )),
-			m_view,SLOT(addTorrent(kt::TorrentInterface* )));
+			m_view_man,SLOT(addTorrent(kt::TorrentInterface* )));
 
 	connect(m_core,SIGNAL(torrentRemoved(kt::TorrentInterface* )),
-			m_view,SLOT(removeTorrent(kt::TorrentInterface* )));
+			m_view_man,SLOT(removeTorrent(kt::TorrentInterface* )));
 
-	connect(m_view,SIGNAL(currentChanged(kt::TorrentInterface* )),
-			this,SLOT(currentTorrentChanged(kt::TorrentInterface* )));
-
-	connect(m_view,SIGNAL(wantToRemove(kt::TorrentInterface*,bool )),
-			m_core,SLOT(remove(kt::TorrentInterface*,bool )));
-
-	connect(m_view,SIGNAL(dropped(QDropEvent*,QListViewItem*)),
-			this,SLOT(urlDropped(QDropEvent*,QListViewItem*)));
-	
-	connect(m_view,SIGNAL(wantToStart( kt::TorrentInterface* )),
-			m_core,SLOT(start( kt::TorrentInterface* )));
-	
-	connect(m_view,SIGNAL(wantToStop( kt::TorrentInterface*, bool )),
-			m_core,SLOT(stop( kt::TorrentInterface*, bool )));
-	
-	connect(m_view,SIGNAL(needsDataCheck( kt::TorrentInterface* )),
-			m_core,SLOT(doDataCheck( kt::TorrentInterface* )));
-	
-	connect(m_view,SIGNAL(updateActions( bool, bool, bool, bool )),
-			this,SLOT(onUpdateActions( bool, bool, bool, bool )));
-	
-	//connect Core queue() with queue() from KTView.
-	connect(m_view, SIGNAL(queue( kt::TorrentInterface* )), 
-			m_core, SLOT(queue( kt::TorrentInterface* )));
-	
 	connect(m_core, SIGNAL(torrentRemoved( kt::TorrentInterface* )),
 			m_group_view, SLOT(onTorrentRemoved( kt::TorrentInterface* )));
 	
-
 	m_statusInfo = new KSqueezedTextLabel(this);
 	m_statusSpeed = new QLabel(this);
 	m_statusTransfer = new QLabel(this);
 	m_statusDHT = new QLabel(this);
-	
-
+	m_statusFirewall = new QLabel(this);
+	m_statusFirewall->setPixmap(SmallIcon("messagebox_warning"));
+	QToolTip::add(m_statusFirewall,i18n("No incoming connections (possibly firewalled)"));
 
+	statusBar()->addWidget(m_statusFirewall);
 	statusBar()->addWidget(m_statusInfo,1);
 	statusBar()->addWidget(m_statusDHT);
 	statusBar()->addWidget(m_statusSpeed);
 	statusBar()->addWidget(m_statusTransfer);
 
-	
+	m_statusFirewall->hide(); 
+
 	setupActions();
 	currentTorrentChanged(0);
 	
@@ -219,13 +210,6 @@ KTorrent::KTorrent()
 	}
 	m_gui_update_timer.start(val);
 	
-	groupChanged(m_group_view->currentGroup());
-	tabWidget()->setHoverCloseButton(false);
-	
-	m_close_cur_tab = new KPushButton(KGuiItem(QString::null,"tab_remove"),tabWidget());
-	connect(m_close_cur_tab,SIGNAL(clicked()),this,SLOT(tabClosePressed()));
-	tabWidget()->setCornerWidget(m_close_cur_tab,Qt::TopRight);
-	connect(tabWidget(),SIGNAL(currentChanged(QWidget*)),this,SLOT(currentTabChanged( QWidget* )));
 	statusBar()->show();
 	
 	addToolWidget(m_group_view,"player_playlist",i18n("Groups"),DOCK_LEFT);
@@ -249,62 +233,140 @@ KTorrent::KTorrent()
 	{
 		show();
 	}
+
+	bool menubar_hidden = KGlobal::config()->readBoolEntry("menubar_hidden",false);
+	menuBar()->setHidden(menubar_hidden);
+	m_menubarAction->setChecked(!menubar_hidden);
+
+	bool statusbar_hidden = KGlobal::config()->readBoolEntry("statusbar_hidden",false);
+	statusBar()->setHidden(statusbar_hidden);
+	m_statusbarAction->setChecked(!statusbar_hidden);
+
 	MaximizeLimits();
+	connect(&m_status_msg_expire,SIGNAL(timeout()),this,SLOT(statusBarMsgExpired()));
 }
 
 KTorrent::~KTorrent()
 {
-	while (m_tab_map.count() > 0)
-	{
-		QMap<QWidget*,Tab>::iterator itr = m_tab_map.begin();
-		removeTabPage(itr.key());
-	}
 	delete m_dcop;
 	delete m_core;
 	delete m_pref;
 	delete m_statusInfo;
 	delete m_statusTransfer;
 	delete m_statusSpeed;
-	delete m_view_exp;
+	delete m_statusFirewall;
+}
+
+void KTorrent::openView(kt::Group* g)
+{
+	if (!g)
+		return;
+	
+	KTorrentView* v = m_view_man->newView();
+	v->setCurrentGroup((Group*)g);
+	v->setupViewColumns();
+	
+	addTabPage(v,g->groupIcon(),v->caption(),m_view_man);
+	
+	connect(v,SIGNAL(currentChanged(kt::TorrentInterface* )),
+			this,SLOT(currentTorrentChanged(kt::TorrentInterface* )));
+
+	connect(v,SIGNAL(wantToRemove(kt::TorrentInterface*,bool )),
+			m_core,SLOT(remove(kt::TorrentInterface*,bool )));
+
+	connect(v->listView(),SIGNAL(dropped(QDropEvent*,QListViewItem*)),
+			this,SLOT(urlDropped(QDropEvent*,QListViewItem*)));
+	
+	connect(v,SIGNAL(wantToStart( kt::TorrentInterface* )),
+			m_core,SLOT(start( kt::TorrentInterface* )));
+	
+	connect(v,SIGNAL(wantToStop( kt::TorrentInterface*, bool )),
+			m_core,SLOT(stop( kt::TorrentInterface*, bool )));
+	
+	connect(v,SIGNAL(needsDataCheck( kt::TorrentInterface* )),
+			m_core,SLOT(doDataCheck( kt::TorrentInterface* )));
+	
+	connect(v,SIGNAL(updateActions( bool, bool, bool, bool )),
+			this,SLOT(onUpdateActions( bool, bool, bool, bool )));
+	
+	//connect Core queue() with queue() from KTView.
+	connect(v, SIGNAL(queue( kt::TorrentInterface* )), 
+			m_core, SLOT(queue( kt::TorrentInterface* )));
+	
+	connect(v,SIGNAL(updateGroupsSubMenu(KPopupMenu*)),
+			m_group_view,SLOT(updateGroupsSubMenu(KPopupMenu*)));
+	
+	connect(v,SIGNAL(groupsSubMenuItemActivated(KTorrentView*, const QString&)),
+			m_group_view,SLOT(onGroupsSubMenuItemActivated(KTorrentView*, const QString&)));
+	
+	if (m_core)
+	{
+		QueueManager* qman = m_core->getQueueManager();
+		QueueManager::iterator i = qman->begin();
+		while (i != qman->end())
+		{
+			v->addTorrent(*i);
+			i++;
+		}
+	}
+}
+
+void KTorrent::openView(const QString & group_name)
+{
+	const kt::Group* g = m_group_view->findGroup(group_name);
+	if (g)
+		openView((kt::Group*)g);
 }
 
 void KTorrent::groupChanged(kt::Group* g)
 {
-	if (!m_tab_map.contains(m_view_exp))
-		return;
-	
-	KMdiChildView* cv = m_tab_map[m_view_exp].child;
-	if (cv)
+	KTorrentView* v = m_view_man->getCurrentView();
+	if (v)
 	{
-		cv->setCaption(g->groupName());
-		cv->setMDICaption(g->groupName());
-		cv->setTabCaption(g->groupName());
-		cv->setIcon(g->groupIcon());
+		m_activeTabWidget->changeTab(v,g->groupName());
+		v->setIcon(g->groupIcon());
+		v->setCurrentGroup(g);
 	}
-	m_view->setCurrentGroup(g);
+}
+
+void KTorrent::groupRenamed(kt::Group* g)
+{
+	m_view_man->groupRenamed(g,m_activeTabWidget);
+}
+
+void KTorrent::groupRemoved(kt::Group* g)
+{
+	kt::Group* allg = m_group_view->groupManager()->allGroup();
+	m_view_man->groupRemoved(g,m_activeTabWidget,this,allg);
 }
 
 void KTorrent::addTabPage(QWidget* page,const QIconSet & icon,
 						  const QString & caption,kt::CloseTabListener* ctl)
 {
-	KMdiChildView* cv = createWrapper(page,caption,caption);
-	addWindow(cv);
-	cv->setIcon(icon.pixmap(QIconSet::Small,QIconSet::Active));
-	Tab t;
-	t.child = cv;
-	t.ctl = ctl;
-	m_tab_map.insert(page,t);
+	addWidget(page,caption);
+	page->setIcon(icon.pixmap(QIconSet::Small,QIconSet::Active));
+	m_tab_map[page] = ctl;
+	currentTabChanged(page);
 }
 
+void KTorrent::addTabPage(QWidget* page,const QPixmap & icon,
+						  const QString & caption,kt::CloseTabListener* ctl)
+{
+	addWidget(page,caption);
+	page->setIcon(icon);
+	m_tab_map[page] = ctl;
+	currentTabChanged(page);
+}
+
+
 void KTorrent::removeTabPage(QWidget* page)
 {
 	if (!m_tab_map.contains(page))
 		return;
 	
-	KMdiChildView* cv = m_tab_map[page].child;
 	m_tab_map.erase(page);
 	page->reparent(0,QPoint());
-	closeWindow(cv);
+	removeWidget(page);
 }
 
 void KTorrent::addPrefPage(PrefPageInterface* page)
@@ -323,12 +385,12 @@ void KTorrent::applySettings(bool change_port)
 	m_core->setMaxSeeds(Settings::maxSeeds());
 	PeerManager::setMaxConnections(Settings::maxConnections());
 	PeerManager::setMaxTotalConnections(Settings::maxTotalConnections());
-//	UploadCap::instance().setMaxSpeed(Settings::maxUploadRate() * 1024);
-//	DownloadCap::instance().setMaxSpeed(Settings::maxDownloadRate()*1024);
 	net::SocketMonitor::setDownloadCap(Settings::maxDownloadRate()*1024);
 	net::SocketMonitor::setUploadCap(Settings::maxUploadRate()*1024);
+	net::SocketMonitor::setSleepTime(Settings::cpuUsage());
 	m_core->setKeepSeeding(Settings::keepSeeding());
-	mse::StreamSocket::setTOS(Settings::typeOfService());
+	mse::StreamSocket::setTOS(Settings::dSCP() << 2);
+	mse::StreamSocket::setMaxConnecting(Settings::maxConnectingSockets());
 	if (Settings::allwaysDoUploadDataCheck())
 		ChunkManager::setMaxChunkSizeForDataCheck(0);
 	else
@@ -381,7 +443,7 @@ void KTorrent::applySettings(bool change_port)
 	dht::DHTBase & ht = Globals::instance().getDHT();
 	if (Settings::dhtSupport() && !ht.isRunning())
 	{
-		ht.start(kt::DataDir() + "dht_table",Settings::dhtPort());
+		ht.start(kt::DataDir() + "dht_table",kt::DataDir() + "dht_key",Settings::dhtPort());
 	}
 	else if (!Settings::dhtSupport() && ht.isRunning())
 	{
@@ -391,7 +453,7 @@ void KTorrent::applySettings(bool change_port)
 	{
 		Out(SYS_GEN|LOG_NOTICE) << "Restarting DHT with new port " << Settings::dhtPort() << endl;
 		ht.stop();
-		ht.start(kt::DataDir() + "dht_table",Settings::dhtPort());
+		ht.start(kt::DataDir() + "dht_table",kt::DataDir() + "dht_key",Settings::dhtPort());
 	}
 	
 	if (Settings::useEncryption())
@@ -438,6 +500,7 @@ void KTorrent::setupActions()
 	KStdAction::paste(kapp,SLOT(paste()),actionCollection());
 
 	m_statusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()), actionCollection());
+	m_menubarAction = KStdAction::showMenubar(this, SLOT(optionsShowMenubar()), actionCollection()); 
 
 	KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
 	KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
@@ -475,7 +538,7 @@ void KTorrent::setupActions()
 	
 	m_queueaction = new KAction(
 			i18n("Enqueue/Dequeue"),
-			"player_playlist", 0, this, SLOT(queueAction()),
+			"player_playlist", 0, m_view_man, SLOT(queueAction()),
 			actionCollection(), "queue_action");
 	
 	m_ipfilter = new KAction(
@@ -485,7 +548,9 @@ void KTorrent::setupActions()
 	
 	m_datacheck = new KAction(
 			i18n("Check Data Integrity"),
-	QString::null,0,this,SLOT(checkDataIntegrity()),actionCollection(),"check_data");
+	QString::null,0,m_view_man,SLOT(checkDataIntegrity()),actionCollection(),"check_data");
+	
+	m_find = KStdAction::find(this,SLOT(find()),actionCollection());
 	
 	//Plug actions to systemtray context menu
 	m_startall->plug(m_systray_icon->contextMenu());
@@ -522,17 +587,17 @@ bool KTorrent::queryClose()
 
 bool KTorrent::queryExit()
 {
-#ifdef KT_PROFILE
-	bt::Profiler::instance().saveToFile(kt::DataDir() + "profile");
-#endif
-	m_group_view->saveGroups();
 	// stop timers to prevent update
 	m_gui_update_timer.stop();
 	
 	KGlobal::config()->setGroup("WindowStatus");
-	KGlobal::config()->writeEntry( "hidden_on_exit",this->isHidden());
-	m_view->saveSettings();
+	KGlobal::config()->writeEntry("hidden_on_exit",this->isHidden());
+	KGlobal::config()->writeEntry("menubar_hidden",menuBar()->isHidden());
+	KGlobal::config()->writeEntry("statusbar_hidden",statusBar()->isHidden());
+	m_view_man->saveViewState(KGlobal::config());
+	saveSettings();
 	hide();
+	m_systray_icon->hide(); // hide system tray icon upon exit
 	m_core->onExit();
 	if (Globals::instance().getDHT().isRunning())
 		Globals::instance().getDHT().stop();
@@ -550,7 +615,6 @@ void KTorrent::fileNew()
 
 void KTorrent::fileOpen()
 {
-
 	QString filter = "*.torrent|" + i18n("Torrent Files") + "\n*|" + i18n("All Files");
 	KURL url = KFileDialog::getOpenURL(QString::null, filter, this, i18n("Open Location"));
 
@@ -572,42 +636,32 @@ void KTorrent::queueManagerShow()
 
 void KTorrent::startDownload()
 {
-	m_view->startDownloads();
-	TorrentInterface* tc = m_view->getCurrentTC();
-	
+	m_view_man->startDownloads();
+	TorrentInterface* tc = m_view_man->getCurrentTC();
 	currentTorrentChanged(tc);
-	m_view->onSelectionChanged(); // trigger an updateActions signal
 }
 
 void KTorrent::startAllDownloads()
 {
-	m_view->startAllDownloads();
-	m_view->onSelectionChanged(); // trigger an updateActions signal
+	m_view_man->startAllDownloads();
 }
 
 void KTorrent::stopDownload()
 {
-	m_view->stopDownloads();
-			
-	TorrentInterface* tc = m_view->getCurrentTC();	
+	m_view_man->stopDownloads();
+	TorrentInterface* tc = m_view_man->getCurrentTC();	
 	currentTorrentChanged(tc);
-	m_view->onSelectionChanged(); // trigger an updateActions signal
 }
 
 void KTorrent::stopAllDownloads()
 {
-	m_view->stopAllDownloads();
-	m_view->onSelectionChanged(); // trigger an updateActions signal
+	m_view_man->stopAllDownloads();
 }
 
 void KTorrent::removeDownload()
 {
-	m_view->removeDownloads();
-			
-	TorrentInterface* tc = m_view->getCurrentTC();
-	currentTorrentChanged(tc);
-	notifyViewListeners(tc);
-	m_view->onSelectionChanged(); // trigger an updateActions signal
+	m_view_man->removeDownloads();
+	currentTorrentChanged(m_view_man->getCurrentTC());
 }
 
 void KTorrent::optionsShowStatusbar()
@@ -620,6 +674,16 @@ void KTorrent::optionsShowStatusbar()
 		statusBar()->hide();
 }
 
+void KTorrent::optionsShowMenubar()
+{
+	// this is all very cut and paste code for showing/hiding the
+	// menubar
+	if (m_menubarAction->isChecked())
+		menuBar()->show();
+	else
+		menuBar()->hide();
+}
+
 void KTorrent::optionsConfigureKeys()
 {
 	KKeyDialog::configure(actionCollection());
@@ -671,6 +735,8 @@ void KTorrent::changeStatusbar(const QString& text)
 	// display the text on the statusbar
 	//statusBar()->message(text);
 	m_statusInfo->setText(text);
+	m_status_msg_expire.stop();
+	m_status_msg_expire.start(5000,true);
 }
 
 void KTorrent::changeCaption(const QString& text)
@@ -681,22 +747,10 @@ void KTorrent::changeCaption(const QString& text)
 
 void KTorrent::saveProperties(KConfig* )
 {
-	KGlobal::config()->writeEntry( "hidden_on_exit",this->isHidden());
-	KGlobal::config()->sync();
 }
 
 void KTorrent::readProperties(KConfig*)
 {
-	bool hidden_on_exit = KGlobal::config()->readBoolEntry("hidden_on_exit",false);
-	if (!(Settings::showSystemTrayIcon() && hidden_on_exit))
-	{
-		Out(SYS_GEN|LOG_DEBUG) << "Showing KT" << endl;
-		show();
-	}
-	else
-	{
-		hide();
-	}
 }
 
 void KTorrent::urlDropped(QDropEvent* event,QListViewItem*)
@@ -712,8 +766,6 @@ void KTorrent::urlDropped(QDropEvent* event,QListViewItem*)
 
 void KTorrent::updatedStats()
 {
-	KT_PROF_START("updatedStats");
-	KT_PROF_START("misc");
 	m_startall->setEnabled(m_core->getNumTorrentsNotRunning() > 0);
 	m_stopall->setEnabled(m_core->getNumTorrentsRunning() > 0);
 	
@@ -730,34 +782,19 @@ void KTorrent::updatedStats()
 			.arg(BytesToString(stats.bytes_downloaded))
 			.arg(BytesToString(stats.bytes_uploaded));
 	m_statusTransfer->setText(tmp1);
-	KT_PROF_END();
 	
-	KT_PROF_START("view");
-	m_view->update();
-	KT_PROF_END();
+	if (ServerAuthenticate::isFirewalled() && m_core->getNumTorrentsRunning() > 0) 
+		m_statusFirewall->show();
+	else
+		m_statusFirewall->hide();
+
+	m_view_man->update();
 	
-	KT_PROF_START("systray");
 	m_systray_icon->updateStats(stats,Settings::showSpeedBarInTrayIcon(),Settings::downloadBandwidth(), Settings::uploadBandwidth());
-	KT_PROF_END();
 	
-	KT_PROF_START("plugins");
 	m_core->getPluginManager().updateGuiPlugins();
-	KT_PROF_END();
 	
-#if 0
 
-	//update tab labels
-	QString tabText = i18n("Downloads %1/%2").arg(m_core->getNumRunning(true)).arg(m_core->countDownloads());
-	//kdDebug() << "tabtext: " << tabText << " " << m_tabs->tabLabel(m_view_exp) << endl;
-	if (tabText != tabWidget()->tabLabel(m_view_exp).replace('&', ""))
-		tabWidget()->setTabLabel(m_view_exp, tabText);
-		
-	tabText = i18n("Uploads %1/%2").arg(m_core->getNumRunning(false,true)).arg(m_core->countSeeds());
-	if (tabText != m_tabs->tabLabel(m_seedView_exp).replace('&', ""))
-		m_tabs->setTabLabel(m_seedView_exp, tabText);
-#endif
-
-	KT_PROF_START("DHT");
 	if (Globals::instance().getDHT().isRunning())
 	{
 		const dht::Stats & s = Globals::instance().getDHT().getStats();
@@ -766,8 +803,6 @@ void KTorrent::updatedStats()
 	}
 	else
 		m_statusDHT->setText(i18n("DHT: off"));
-	KT_PROF_END();
-	KT_PROF_END();
 }
 
 void KTorrent::mergePluginGui(Plugin* p)
@@ -784,16 +819,10 @@ void KTorrent::removePluginGui(Plugin* p)
 
 void KTorrent::addWidgetInView(QWidget* w,Position pos)
 {
-	if (!w) return;
-	
-	m_view_exp->expand(w,pos);
 }
 
 void KTorrent::removeWidgetFromView(QWidget* w)
 {
-	if (!w) return;
-	
-	m_view_exp->remove(w);
 }
 
 void KTorrent::addWidgetBelowView(QWidget* w,const QString & icon,const QString & caption)
@@ -810,23 +839,23 @@ void KTorrent::addToolWidget(QWidget* w,const QString & icon,const QString & cap
 {
 	if (!w) return;
 	
-	KMdiToolViewAccessor* tw = createToolWindow();
+	
 	if (!icon.isNull())
 		w->setIcon(KGlobal::iconLoader()->loadIcon(icon,KIcon::Small));
-	tw->setWidgetToWrap(w,caption,caption);
+	
 	switch (dock)
 	{
 		case DOCK_BOTTOM:
-			tw->place(KDockWidget::DockBottom,getMainDockWidget(),25);
+			addDockWidget(DDockWindow::Bottom,w,caption);
 			break;
 			
 		case DOCK_LEFT:
-			tw->place(KDockWidget::DockLeft,getMainDockWidget(),25);
+			addDockWidget(DDockWindow::Left,w,caption);
 			break;
 			
 		case DOCK_RIGHT:
 		default:
-			tw->place(KDockWidget::DockRight,getMainDockWidget(),25);
+			addDockWidget(DDockWindow::Right,w,caption);
 			break;
 	}
 }
@@ -835,23 +864,13 @@ void KTorrent::removeToolWidget(QWidget* w)
 {
 	if (!w) return;
 	
+	removeDockWidget(w);
 	w->reparent(0,QPoint());
-	deleteToolWindow(w);
 }
 
 const TorrentInterface* KTorrent::getCurrentTorrent() const
 {
-	return m_view->getCurrentTC();
-}
-
-void KTorrent::queueAction()
-{
-	m_view->queueSlot();
-}
-
-void KTorrent::checkDataIntegrity()
-{
-	m_view->checkDataIntegrity();
+	return m_view_man->getCurrentTC();
 }
 
 QString KTorrent::getStatusInfo() 
@@ -874,9 +893,14 @@ QString KTorrent::getStatusDHT()
 	return m_statusDHT->text();
 }
 
+QString KTorrent::getStatusFirewall()
+{
+	return m_statusFirewall->text();
+}
+
 QCStringList KTorrent::getTorrentInfo(kt::TorrentInterface* tc)
 {
-	return m_view->getTorrentInfo(tc);
+	return KTorrentViewItem::getTorrentInfo(tc);
 }
 
 void KTorrent::showIPFilter()
@@ -885,42 +909,90 @@ void KTorrent::showIPFilter()
 	ipf.exec();
 }
 
-void KTorrent::tabClosePressed()
+
+void KTorrent::closeTab()
 {
-	KMdiChildView* child = activeWindow();
-	if (!child)
+	QWidget* w = m_currentWidget;
+	if (!w)
 		return;
 	
-	QMap<QWidget*,Tab>::iterator i = m_tab_map.begin();
-	while (i != m_tab_map.end())
+	CloseTabListener* ctl = m_tab_map[w];
+	if (ctl)
 	{
-		if (i.data().child == child)
-		{
-			kt::CloseTabListener* ctl = i.data().ctl;
-			ctl->tabCloseRequest(i.key());
-			return; 
-		}
-		i++;
+		ctl->tabCloseRequest(this,w);
+		currentTabChanged(m_activeTabWidget->currentPage());
 	}
 }
 
-void KTorrent::currentTabChanged(QWidget* )
+void KTorrent::currentTabChanged(QWidget* w)
 {
-	KMdiChildView* child = activeWindow();
-	if (!child)
+	m_view_man->onCurrentTabChanged(w);
+	currentTorrentChanged(m_view_man->getCurrentTC());
+	if (!m_activeTabWidget || !w)
 		return;
 	
-	QMap<QWidget*,Tab>::iterator i = m_tab_map.begin();
-	while (i != m_tab_map.end())
-	{
-		if (i.data().child == child)
-		{
-			m_close_cur_tab->setEnabled(i.data().ctl != 0);
-			return;
-		}
-		i++;
-	}
-	m_close_cur_tab->setEnabled(false);
+	bool close_allowed = false;
+	CloseTabListener* ctl = m_tab_map[w];
+	if (ctl)
+		close_allowed = ctl->closeAllowed(w);
+	
+	m_activeTabWidget->closeButton()->setEnabled(close_allowed);
+}
+
+void KTorrent::openDefaultView()
+{
+	openView(i18n("All Torrents"));
+}
+
+KToolBar* KTorrent::addToolBar(const char* name)
+{
+	return toolBar(name);
+}
+
+void KTorrent::removeToolBar(KToolBar* tb)
+{
+	delete tb;
+}
+
+void KTorrent::loadSilentlyDir(const KURL& url, const KURL& savedir)
+{
+	m_core->loadSilentlyDir(url, savedir);
+}
+
+KProgress* KTorrent::addProgressBarToStatusBar()
+{
+	if (m_status_prog)
+		return 0;
+	
+	KStatusBar* sb = statusBar();
+	m_status_prog = new KProgress(100,sb);
+	m_status_prog->setValue(0);
+	sb->addWidget(m_status_prog);
+	m_status_prog->show();
+	return m_status_prog;
+}
+
+void KTorrent::removeProgressBarFromStatusBar(KProgress* p)
+{
+	if (m_status_prog != p)
+		return;
+	
+	KStatusBar* sb = statusBar();
+	sb->removeWidget(p);
+	delete p;
+	m_status_prog = 0;
+}
+
+void KTorrent::statusBarMsgExpired()
+{
+	m_statusInfo->clear();
+}
+
+void KTorrent::find()
+{
+	KTorrentView* v = m_view_man->getCurrentView();
+	if (v)
+		v->toggleFilterBar();
 }
 
 #include "ktorrent.moc"
