@@ -29,6 +29,8 @@
 #include "ktorrentcore.h"
 #include <interfaces/functions.h>
 #include <net/socketmonitor.h>
+#include <util/log.h>
+#include "trayhoverpopup.h"
 
 
 using namespace bt;
@@ -38,11 +40,14 @@ TrayIcon::TrayIcon( KTorrentCore* tc, QWidget *parent, const char *name)
 		: KSystemTray(parent, name)
 {
 	m_core = tc;
-	setPixmap(loadIcon("ktorrent"));
+	m_kt_pix = loadIcon("ktorrent");
+	setPixmap(m_kt_pix);
 	paint=new QPainter( this );
 	drawContents ( paint );
 	previousDownloadHeight=0;
 	previousUploadHeight=0;
+	
+	m_hover_popup = new TrayHoverPopup(m_kt_pix,this);
 
 	connect(this,SIGNAL(quitSelected()),kapp,SLOT(quit()));
 	connect(m_core, SIGNAL(finished(kt::TorrentInterface* )),
@@ -51,20 +56,44 @@ TrayIcon::TrayIcon( KTorrentCore* tc, QWidget *parent, const char *name)
 	        this,SLOT(torrentStoppedByError(kt::TorrentInterface*, QString )));
 	connect(m_core,SIGNAL(maxShareRatioReached( kt::TorrentInterface* )),
 			this,SLOT(maxShareRatioReached( kt::TorrentInterface* )));
+	connect(m_core,SIGNAL(maxSeedTimeReached(kt::TorrentInterface*)),
+			this, SLOT(maxSeedTimeReached(kt::TorrentInterface*)));
 	connect(m_core,SIGNAL(corruptedData( kt::TorrentInterface* )),
 			this,SLOT(corruptedData( kt::TorrentInterface* )));
 	connect(m_core, SIGNAL(queuingNotPossible( kt::TorrentInterface* )),
-			this, SLOT(queuedTorrentOverMaxRatio( kt::TorrentInterface* )));
+			this, SLOT(queuingNotPossible( kt::TorrentInterface* )));
+	connect(m_core,SIGNAL(canNotStart(kt::TorrentInterface*, kt::TorrentStartResponse)),
+			this,SLOT(canNotStart(kt::TorrentInterface*, kt::TorrentStartResponse)));
+	connect(m_core, SIGNAL(lowDiskSpace(kt::TorrentInterface*, bool)),
+			this, SLOT(lowDiskSpace(kt::TorrentInterface*, bool)));
+	
+	connect(this->contextMenu(),SIGNAL(aboutToShow()),m_hover_popup,SLOT(contextMenuAboutToShow()));
+	connect(this->contextMenu(),SIGNAL(aboutToHide()),m_hover_popup,SLOT(contextMenuAboutToHide()));
 }
 
 TrayIcon::~TrayIcon()
 {}
 
+void TrayIcon::enterEvent(QEvent* ev)
+{
+	KSystemTray::enterEvent(ev);
+	m_hover_popup->enterEvent();
+}
+
+void TrayIcon::leaveEvent(QEvent* )
+{
+	m_hover_popup->leaveEvent();
+}
+
 void TrayIcon::updateStats(const CurrentStats stats, bool showBars,int downloadBandwidth, int uploadBandwidth )
 {
-	QString tip = i18n("<center><b>KTorrent</b></center><table cellpadding='2' cellspacing='2' align='center'><tr><td><b>Speed:</b></td><td></td></tr><tr><td>Download: <font color='#1c9a1c'>%1</font></td><td>Upload: <font color='#990000'>%2</font></td></tr><tr><td><b>Transfer:</b></td><td></td></tr><tr><td>Download: <font color='#1c9a1c'>%3</font></td><td>Upload: <font color='#990000'>%4</font></td></tr></table>").arg(KBytesPerSecToString((double)stats.download_speed/1024.0)).arg(KBytesPerSecToString((double)stats.upload_speed/1024.0)).arg(BytesToString(stats.bytes_downloaded)).arg(BytesToString(stats.bytes_uploaded));
-	QToolTip::add
-		(this, tip);
+	QString tip = i18n("<table cellpadding='2' cellspacing='2' align='center'><tr><td><b>Speed:</b></td><td></td></tr><tr><td>Download: <font color='#1c9a1c'>%1</font></td><td>Upload: <font color='#990000'>%2</font></td></tr><tr><td><b>Transfer:</b></td><td></td></tr><tr><td>Download: <font color='#1c9a1c'>%3</font></td><td>Upload: <font color='#990000'>%4</font></td></tr></table>")
+			.arg(KBytesPerSecToString((double)stats.download_speed/1024.0))
+			.arg(KBytesPerSecToString((double)stats.upload_speed/1024.0))
+			.arg(BytesToString(stats.bytes_downloaded))
+			.arg(BytesToString(stats.bytes_uploaded));
+	m_hover_popup->updateText(tip);
+	
 	if(showBars)
 		drawSpeedBar(stats.download_speed/1024,stats.upload_speed/1024, downloadBandwidth, uploadBandwidth);
 }
@@ -93,6 +122,14 @@ void TrayIcon::drawSpeedBar(int downloadSpeed, int uploadSpeed, int downloadBand
 	previousUploadHeight=UploadHeight;	
 }
 
+void TrayIcon::showPassivePopup(const QString & msg,const QString & title)
+{
+	KPassivePopup* p = KPassivePopup::message(KPassivePopup::Boxed,title,msg,m_kt_pix, this);
+	p->setPalette(QToolTip::palette());
+	p->setLineWidth(1);
+}
+
+
 void TrayIcon::finished(TorrentInterface* tc)
 {
 	if (!Settings::showPopups())
@@ -108,7 +145,7 @@ void TrayIcon::finished(TorrentInterface* tc)
 						.arg(KBytesPerSecToString(speed_down / tc->getRunningTimeDL()))
 						.arg(KBytesPerSecToString(speed_up / tc->getRunningTimeUL()));
 
-	KPassivePopup::message(i18n("Download completed"),msg,loadIcon("ktorrent"), this);
+	showPassivePopup(msg,i18n("Download completed"));
 }
 
 void TrayIcon::maxShareRatioReached(kt::TorrentInterface* tc)
@@ -117,18 +154,36 @@ void TrayIcon::maxShareRatioReached(kt::TorrentInterface* tc)
 		return;
 	
 	const TorrentStats & s = tc->getStats();
-	
+	KLocale* loc = KGlobal::locale();
 	double speed_up = (double)s.bytes_uploaded / 1024.0;
 	
 	QString msg = i18n("<b>%1</b> has reached its maximum share ratio of %2 and has been stopped."
 			"<br>Uploaded %3 at an average speed of %4.")
 			.arg(s.torrent_name)
-			.arg(s.max_share_ratio)
+			.arg(loc->formatNumber(s.max_share_ratio,2))
 			.arg(BytesToString(s.bytes_uploaded))
 			.arg(KBytesPerSecToString(speed_up / tc->getRunningTimeUL()));
 	
-	KPassivePopup::message(i18n("Seeding completed"),
-						   msg,loadIcon("ktorrent"), this);
+	showPassivePopup(msg,i18n("Seeding completed"));
+}
+
+void TrayIcon::maxSeedTimeReached(kt::TorrentInterface* tc)
+{
+	if (!Settings::showPopups())
+		return;
+	
+	const TorrentStats & s = tc->getStats();
+	KLocale* loc = KGlobal::locale();
+	double speed_up = (double)s.bytes_uploaded / 1024.0;
+	
+	QString msg = i18n("<b>%1</b> has reached its maximum seed time of %2 hours and has been stopped."
+			"<br>Uploaded %3 at an average speed of %4.")
+			.arg(s.torrent_name)
+			.arg(loc->formatNumber(s.max_seed_time,2))
+			.arg(BytesToString(s.bytes_uploaded))
+			.arg(KBytesPerSecToString(speed_up / tc->getRunningTimeUL()));
+	
+	showPassivePopup(msg,i18n("Seeding completed"));
 }
 
 void TrayIcon::torrentStoppedByError(kt::TorrentInterface* tc, QString msg)
@@ -139,7 +194,8 @@ void TrayIcon::torrentStoppedByError(kt::TorrentInterface* tc, QString msg)
 	const TorrentStats & s = tc->getStats();
 	QString err_msg = i18n("<b>%1</b> has been stopped with the following error: <br>%2")
 				.arg(s.torrent_name).arg(msg);
-	KPassivePopup::message(i18n("Error"),err_msg,loadIcon("ktorrent"),this);
+	
+	showPassivePopup(err_msg,i18n("Error"));
 }
 
 void TrayIcon::corruptedData(kt::TorrentInterface* tc)
@@ -151,22 +207,74 @@ void TrayIcon::corruptedData(kt::TorrentInterface* tc)
 	QString err_msg = i18n("Corrupted data has been found in the torrent <b>%1</b>"
 			"<br>It would be a good idea to do a data integrity check on the torrent.")
 			.arg(s.torrent_name);
-	KPassivePopup::message(i18n("Error"),err_msg,loadIcon("ktorrent"),this);
+	showPassivePopup(err_msg,i18n("Error"));
 }
 
-void TrayIcon::queuedTorrentOverMaxRatio(kt::TorrentInterface* tc)
+void TrayIcon::queuingNotPossible(kt::TorrentInterface* tc)
+{
+	if (!Settings::showPopups())
+		return;
+	
+	const TorrentStats & s = tc->getStats();
+		
+	QString msg;
+	KLocale* loc = KGlobal::locale();
+	 
+	if (tc->overMaxRatio())
+		msg = i18n("<b>%1</b> has reached its maximum share ratio of %2 and cannot be enqueued. Remove the limit manually if you want to continue seeding.")
+				.arg(s.torrent_name).arg(loc->formatNumber(s.max_share_ratio,2));
+	else
+		msg = i18n("<b>%1</b> has reached its maximum seed time of %2 hours and cannot be enqueued. Remove the limit manually if you want to continue seeding.")
+				.arg(s.torrent_name).arg(loc->formatNumber(s.max_seed_time,2));
+	
+	showPassivePopup(msg,i18n("Torrent cannot be enqueued."));
+}
+
+void TrayIcon::canNotStart(kt::TorrentInterface* tc,kt::TorrentStartResponse reason)
+{
+	if (!Settings::showPopups())
+		return;
+	
+	QString msg = i18n("Cannot start <b>%1</b> : <br>").arg(tc->getStats().torrent_name);
+	switch (reason)
+	{
+	case kt::QM_LIMITS_REACHED:
+		if (tc->getStats().bytes_left_to_download == 0)
+		{
+			// is a seeder
+			msg += i18n("Cannot seed more than 1 torrent. <br>",
+						"Cannot seed more than %n torrents. <br>",Settings::maxSeeds());
+		}
+		else
+		{
+			msg += i18n("Cannot download more than 1 torrent. <br>",
+						"Cannot download more than %n torrents. <br>",Settings::maxDownloads());
+		}
+		msg += i18n("Go to Settings -> Configure KTorrent, if you want to change the limits.");
+		showPassivePopup(msg,i18n("Torrent cannot be started"));
+		break;
+	case kt::NOT_ENOUGH_DISKSPACE:
+		msg += i18n("There is not enough diskspace available.");
+		showPassivePopup(msg,i18n("Torrent cannot be started"));
+		break;
+	default:
+		break;
+	}
+}
+
+void TrayIcon::lowDiskSpace(kt::TorrentInterface * tc, bool stopped)
 {
 	if (!Settings::showPopups())
 		return;
 	
 	const TorrentStats & s = tc->getStats();
 	
-	double speed_up = (double)s.bytes_uploaded / 1024.0;
+	QString msg = i18n("Your disk is running out of space.<br /><b>%1</b> is being downloaded to '%2'.").arg(s.torrent_name).arg(tc->getDataDir());
 	
-	QString msg = i18n("<b>%1</b> has reached its maximum share ratio of %2 and cannot be enqueued. Remove the limit manually if you want to continue seeding.").arg(s.torrent_name).arg(s.max_share_ratio);
+	if(stopped)
+		msg.prepend(i18n("Torrent has been stopped.<br />"));
 	
-	KPassivePopup::message(i18n("Torrent cannot be enqueued."),
-						   msg,loadIcon("ktorrent"), this);
+	showPassivePopup(msg,i18n("Device running out of space"));
 }
 
 SetMaxRate::SetMaxRate( KTorrentCore* tc, int t, QWidget *parent, const char *name):KPopupMenu(parent, name)
@@ -242,10 +350,12 @@ void SetMaxRate::update()
 void SetMaxRate::rateSelected(int id)
 {
 	int rate;
-	if(text(id).contains(i18n("Unlimited")))
-		rate=0;
+	QString ratestr = text(id).remove('&');
+	if (ratestr.contains(i18n("Unlimited")))
+		rate = 0;
 	else
-		rate=text(id).toInt();
+		rate = ratestr.toInt();
+
 	if(type==0)
 	{
 		m_core->setMaxUploadSpeed(rate);
@@ -261,4 +371,5 @@ void SetMaxRate::rateSelected(int id)
 	update();
 }
 
+
 #include "trayicon.moc"
